@@ -19,8 +19,26 @@ import logging
 import six
 import sys
 
+from cliff import lister
 from cliff import show
 
+from openstackclient.common import utils
+
+
+class ListCommand(lister.Lister):
+    """List recognized commands by group"""
+
+    auth_required = False
+    log = logging.getLogger(__name__ + '.ListCommand')
+
+    @utils.log_method(log)
+    def take_action(self, parsed_args):
+        cm = self.app.command_manager
+        groups = cm.get_command_groups()
+
+        columns = ('Command Group', 'Commands')
+        return (columns, ((c, cm.get_command_names(group=c)) for c in groups))
+
 
 class ListModule(show.ShowOne):
     """List module versions"""
@@ -38,8 +56,8 @@ class ListModule(show.ShowOne):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         data = {}
         # Get module versions
