@@ -23,6 +23,8 @@ import sys
 from cliff import command
 from cliff import show
 
+from openstackclient.common import utils
+
 
 # List the quota items, map the internal argument name to the option
 # name that the user sees.
@@ -48,6 +50,12 @@ VOLUME_QUOTAS = {
     'volumes': 'volumes',
 }
 
+NETWORK_QUOTAS = {
+    'floatingip': 'floating-ips',
+    'security_group_rule': 'secgroup-rules',
+    'security_group': 'secgroups',
+}
+
 
 class SetQuota(command.Command):
     """Set quotas for project or class"""
@@ -72,27 +80,36 @@ class SetQuota(command.Command):
                 COMPUTE_QUOTAS.items(), VOLUME_QUOTAS.items()):
             parser.add_argument(
                 '--%s' % v,
-                metavar='<new-%s>' % v,
+                metavar='<%s>' % v,
                 type=int,
                 help='New value for the %s quota' % v,
             )
+        parser.add_argument(
+            '--volume-type',
+            metavar='<volume-type>',
+            help='Set quotas for a specific <volume-type>',
+        )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
         volume_client = self.app.client_manager.volume
 
         compute_kwargs = {}
         for k, v in COMPUTE_QUOTAS.items():
-            if v in parsed_args:
-                compute_kwargs[k] = getattr(parsed_args, v, None)
+            value = getattr(parsed_args, k, None)
+            if value is not None:
+                compute_kwargs[k] = value
 
         volume_kwargs = {}
         for k, v in VOLUME_QUOTAS.items():
-            if v in parsed_args:
-                volume_kwargs[k] = getattr(parsed_args, v, None)
+            value = getattr(parsed_args, k, None)
+            if value is not None:
+                if parsed_args.volume_type:
+                    k = k + '_%s' % parsed_args.volume_type
+                volume_kwargs[k] = value
 
         if compute_kwargs == {} and volume_kwargs == {}:
             sys.stderr.write("No quotas updated")
@@ -147,46 +164,67 @@ class ShowQuota(show.ShowOne):
         )
         return parser
 
+    def get_compute_volume_quota(self, client, parsed_args):
+        try:
+            if parsed_args.quota_class:
+                quota = client.quota_classes.get(parsed_args.project)
+            elif parsed_args.default:
+                quota = client.quotas.defaults(parsed_args.project)
+            else:
+                quota = client.quotas.get(parsed_args.project)
+        except Exception as e:
+            if type(e).__name__ == 'EndpointNotFound':
+                return {}
+            else:
+                raise e
+        return quota._info
+
+    def get_network_quota(self, parsed_args):
+        if parsed_args.quota_class or parsed_args.default:
+            return {}
+        service_catalog = self.app.client_manager.auth_ref.service_catalog
+        if 'network' in service_catalog.get_endpoints():
+            network_client = self.app.client_manager.network
+            return network_client.show_quota(parsed_args.project)['quota']
+        else:
+            return {}
+
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
         volume_client = self.app.client_manager.volume
-
         # NOTE(dtroyer): These quota API calls do not validate the project
         #                or class arguments and return what appears to be
         #                the default quota values if the project or class
         #                does not exist. If this is determined to be the
         #                intended behaviour of the API we will validate
         #                the argument with Identity ourselves later.
-        if parsed_args.quota_class:
-            compute_quota = compute_client.quota_classes.get(
-                parsed_args.project)
-            volume_quota = volume_client.quota_classes.get(
-                parsed_args.project)
-        elif parsed_args.default:
-            compute_quota = compute_client.quotas.defaults(
-                parsed_args.project)
-            volume_quota = volume_client.quotas.defaults(
-                parsed_args.project)
-        else:
-            compute_quota = compute_client.quotas.get(parsed_args.project)
-            volume_quota = volume_client.quotas.get(parsed_args.project)
+        compute_quota_info = self.get_compute_volume_quota(compute_client,
+                                                           parsed_args)
+        volume_quota_info = self.get_compute_volume_quota(volume_client,
+                                                          parsed_args)
+        network_quota_info = self.get_network_quota(parsed_args)
 
         info = {}
-        info.update(compute_quota._info)
-        info.update(volume_quota._info)
+        info.update(compute_quota_info)
+        info.update(volume_quota_info)
+        info.update(network_quota_info)
 
         # Map the internal quota names to the external ones
+        # COMPUTE_QUOTAS and NETWORK_QUOTAS share floating-ips,
+        # secgroup-rules and secgroups as dict value, so when
+        # neutron is enabled, quotas of these three resources
+        # in nova will be replaced by neutron's.
         for k, v in itertools.chain(
-                COMPUTE_QUOTAS.items(), VOLUME_QUOTAS.items()):
-            if not k == v and info[k]:
+                COMPUTE_QUOTAS.items(), VOLUME_QUOTAS.items(),
+                NETWORK_QUOTAS.items()):
+            if not k == v and info.get(k):
                 info[v] = info[k]
                 info.pop(k)
 
         # Handle project ID special as it only appears in output
-        if info['id']:
-            info['project'] = info['id']
-            info.pop('id')
+        if 'id' in info:
+            info['project'] = info.pop('id')
 
         return zip(*sorted(six.iteritems(info)))
