@@ -19,30 +19,82 @@ import getpass
 import logging
 import os
 import six
-import sys
 import time
 
+from oslo_utils import importutils
+
 from openstackclient.common import exceptions
-from openstackclient.openstack.common import strutils
 
 
-def find_resource(manager, name_or_id):
-    """Helper for the _find_* methods."""
+def log_method(log, level=logging.DEBUG):
+    """Logs a method and its arguments when entered."""
+
+    def decorator(func):
+        func_name = func.__name__
+
+        @six.wraps(func)
+        def wrapper(self, *args, **kwargs):
+            if log.isEnabledFor(level):
+                pretty_args = []
+                if args:
+                    pretty_args.extend(str(a) for a in args)
+                if kwargs:
+                    pretty_args.extend(
+                        "%s=%s" % (k, v) for k, v in six.iteritems(kwargs))
+                log.log(level, "%s(%s)", func_name, ", ".join(pretty_args))
+            return func(self, *args, **kwargs)
+
+        return wrapper
+
+    return decorator
+
+
+def find_resource(manager, name_or_id, **kwargs):
+    """Helper for the _find_* methods.
+
+    :param manager: A client manager class
+    :param name_or_id: The resource we are trying to find
+    :param kwargs: To be used in calling .find()
+    :rtype: The found resource
+
+    This method will attempt to find a resource in a variety of ways.
+    Primarily .get() methods will be called with `name_or_id` as an integer
+    value, and tried again as a string value.
+
+    If both fail, then a .find() is attempted, which is essentially calling
+    a .list() function with a 'name' query parameter that is set to
+    `name_or_id`.
+
+    Lastly, if any kwargs are passed in, they will be treated as additional
+    query parameters. This is particularly handy in the case of finding
+    resources in a domain.
+
+    """
 
     # Try to get entity as integer id
     try:
         if isinstance(name_or_id, int) or name_or_id.isdigit():
-            return manager.get(int(name_or_id))
-    except exceptions.NotFound:
-        pass
+            return manager.get(int(name_or_id), **kwargs)
+    # FIXME(dtroyer): The exception to catch here is dependent on which
+    #                 client library the manager passed in belongs to.
+    #                 Eventually this should be pulled from a common set
+    #                 of client exceptions.
+    except Exception as ex:
+        if type(ex).__name__ == 'NotFound':
+            pass
+        else:
+            raise
 
     # Try directly using the passed value
     try:
-        return manager.get(name_or_id)
+        return manager.get(name_or_id, **kwargs)
     except Exception:
         pass
 
-    kwargs = {}
+    if len(kwargs) == 0:
+        kwargs = {}
+
+    # Prepare the kwargs for calling find
     if 'NAME_ATTR' in manager.resource_class.__dict__:
         # novaclient does this for oddball resources
         kwargs[manager.resource_class.NAME_ATTR] = name_or_id
@@ -78,12 +130,32 @@ def format_dict(data):
     """
 
     output = ""
-    for s in data:
-        output = output + s + "='" + \
-            strutils.safe_encode(six.text_type(data[s])) + "', "
+    for s in sorted(data):
+        output = output + s + "='" + six.text_type(data[s]) + "', "
     return output[:-2]
 
 
+def format_list(data):
+    """Return a formatted strings
+
+    :param data: a list of strings
+    :rtype: a string formatted to a,b,c
+    """
+
+    return ', '.join(sorted(data))
+
+
+def get_field(item, field):
+    try:
+        if isinstance(item, dict):
+            return item[field]
+        else:
+            return getattr(item, field)
+    except Exception:
+        msg = "Resource doesn't have field %s" % field
+        raise exceptions.CommandError(msg)
+
+
 def get_item_properties(item, fields, mixed_case_fields=[], formatters={}):
     """Return a tuple containing the item properties.
 
@@ -132,6 +204,41 @@ def get_dict_properties(item, fields, mixed_case_fields=[], formatters={}):
     return tuple(row)
 
 
+def sort_items(items, sort_str):
+    """Sort items based on sort keys and sort directions given by sort_str.
+
+    :param items: a list or generator object of items
+    :param sort_str: a string defining the sort rules, the format is
+        '<key1>:[direction1],<key2>:[direction2]...', direction can be 'asc'
+        for ascending or 'desc' for descending, if direction is not given,
+        it's ascending by default
+    :return: sorted items
+    """
+    if not sort_str:
+        return items
+    # items may be a generator object, transform it to a list
+    items = list(items)
+    sort_keys = sort_str.strip().split(',')
+    for sort_key in reversed(sort_keys):
+        reverse = False
+        if ':' in sort_key:
+            sort_key, direction = sort_key.split(':', 1)
+            if not sort_key:
+                msg = "empty string is not a valid sort key"
+                raise exceptions.CommandError(msg)
+            if direction not in ['asc', 'desc']:
+                if not direction:
+                    direction = "empty string"
+                msg = ("%s is not a valid sort direction for sort key %s, "
+                       "use asc or desc instead" % (direction, sort_key))
+                raise exceptions.CommandError(msg)
+            if direction == 'desc':
+                reverse = True
+        items.sort(key=lambda item: get_field(item, sort_key),
+                   reverse=reverse)
+    return items
+
+
 def string_to_bool(arg):
     return arg.strip().lower() in ('t', 'true', 'yes', '1')
 
@@ -149,17 +256,6 @@ def env(*vars, **kwargs):
     return kwargs.get('default', '')
 
 
-def import_class(import_str):
-    """Returns a class from a string including module and class
-
-    :param import_str: a string representation of the class name
-    :rtype: the requested class
-    """
-    mod_str, _sep, class_str = import_str.rpartition('.')
-    __import__(mod_str)
-    return getattr(sys.modules[mod_str], class_str)
-
-
 def get_client_class(api_name, version, version_map):
     """Returns the client class for the requested API version
 
@@ -175,7 +271,7 @@ def get_client_class(api_name, version, version_map):
               (api_name, version, ', '.join(version_map.keys())))
         raise exceptions.UnsupportedVersion(msg)
 
-    return import_class(client_path)
+    return importutils.import_class(client_path)
 
 
 def wait_for_status(status_f,
@@ -210,6 +306,52 @@ def wait_for_status(status_f,
     return retval
 
 
+def wait_for_delete(manager,
+                    res_id,
+                    status_field='status',
+                    sleep_time=5,
+                    timeout=300,
+                    callback=None):
+    """Wait for resource deletion
+
+    :param res_id: the resource id to watch
+    :param status_field: the status attribute in the returned resource object,
+        this is used to check for error states while the resource is being
+        deleted
+    :param sleep_time: wait this long between checks (seconds)
+    :param timeout: check until this long (seconds)
+    :param callback: called per sleep cycle, useful to display progress; this
+        function is passed a progress value during each iteration of the wait
+        loop
+    :rtype: True on success, False if the resource has gone to error state or
+        the timeout has been reached
+    """
+    total_time = 0
+    while total_time < timeout:
+        try:
+            # might not be a bad idea to re-use find_resource here if it was
+            # a bit more friendly in the exceptions it raised so we could just
+            # handle a NotFound exception here without parsing the message
+            res = manager.get(res_id)
+        except Exception as ex:
+            if type(ex).__name__ == 'NotFound':
+                return True
+            raise
+
+        status = getattr(res, status_field, '').lower()
+        if status == 'error':
+            return False
+
+        if callback:
+            progress = getattr(res, 'progress', None) or 0
+            callback(progress)
+        time.sleep(sleep_time)
+        total_time += sleep_time
+
+    # if we got this far we've timed out
+    return False
+
+
 def get_effective_log_level():
     """Returns the lowest logging level considered by logging handlers
 
@@ -223,12 +365,15 @@ def get_effective_log_level():
     return min_log_lvl
 
 
-def get_password(stdin):
+def get_password(stdin, prompt=None, confirm=True):
+    message = prompt or "User Password:"
     if hasattr(stdin, 'isatty') and stdin.isatty():
         try:
             while True:
-                first_pass = getpass.getpass("User password: ")
-                second_pass = getpass.getpass("Repeat user password: ")
+                first_pass = getpass.getpass(message)
+                if not confirm:
+                    return first_pass
+                second_pass = getpass.getpass("Repeat " + message)
                 if first_pass == second_pass:
                     return first_pass
                 print("The passwords entered were not the same")
@@ -246,3 +391,11 @@ def read_blob_file_contents(blob_file):
     except IOError:
         msg = "Error occurred trying to read from file %s"
         raise exceptions.CommandError(msg % blob_file)
+
+
+def build_kwargs_dict(arg_name, value):
+    """Return a dictionary containing `arg_name` if `value` is set."""
+    kwargs = {}
+    if value:
+        kwargs[arg_name] = value
+    return kwargs
