@@ -23,7 +23,13 @@ from cliff import command
 from cliff import lister
 from cliff import show
 
-from novaclient.v1_1 import security_group_rules
+from keystoneclient import exceptions as ksc_exc
+
+try:
+    from novaclient.v2 import security_group_rules
+except ImportError:
+    from novaclient.v1_1 import security_group_rules
+
 from openstackclient.common import parseractions
 from openstackclient.common import utils
 
@@ -44,10 +50,10 @@ def _xform_security_group_rule(sgroup):
         info['ip_range'] = info['ip_range']['cidr']
     else:
         info['ip_range'] = ''
-    if info['ip_protocol'] == 'icmp':
-        info['port_range'] = ''
-    elif info['ip_protocol'] is None:
+    if info['ip_protocol'] is None:
         info['ip_protocol'] = ''
+    elif info['ip_protocol'].lower() == 'icmp':
+        info['port_range'] = ''
     return info
 
 
@@ -75,9 +81,11 @@ class CreateSecurityGroup(show.ShowOne):
 
         compute_client = self.app.client_manager.compute
 
+        description = parsed_args.description or parsed_args.name
+
         data = compute_client.security_groups.create(
             parsed_args.name,
-            parsed_args.description,
+            description,
         )
 
         info = {}
@@ -99,8 +107,8 @@ class DeleteSecurityGroup(command.Command):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
         data = utils.find_resource(
@@ -150,10 +158,15 @@ class ListSecurityGroup(lister.Lister):
         search = {'all_tenants': parsed_args.all_projects}
         data = compute_client.security_groups.list(search_opts=search)
 
-        projects = self.app.client_manager.identity.projects.list()
         project_hash = {}
-        for project in projects:
-            project_hash[project.id] = project
+        try:
+            projects = self.app.client_manager.identity.projects.list()
+        except ksc_exc.ClientException:
+            # This fails when the user is not an admin, just move along
+            pass
+        else:
+            for project in projects:
+                project_hash[project.id] = project
 
         return (column_headers,
                 (utils.get_item_properties(
@@ -186,8 +199,8 @@ class SetSecurityGroup(show.ShowOne):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
         data = utils.find_resource(
@@ -227,8 +240,8 @@ class ShowSecurityGroup(show.ShowOne):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
         info = {}
@@ -279,6 +292,7 @@ class CreateSecurityGroupRule(show.ShowOne):
         parser.add_argument(
             "--dst-port",
             metavar="<port-range>",
+            default=(0, 0),
             action=parseractions.RangeAction,
             help="Destination port, may be a range: 137:139 (default: 0; "
                  "only required for proto tcp and udp)",
@@ -293,7 +307,10 @@ class CreateSecurityGroupRule(show.ShowOne):
             compute_client.security_groups,
             parsed_args.group,
         )
-        from_port, to_port = parsed_args.dst_port
+        if parsed_args.proto.lower() == 'icmp':
+            from_port, to_port = -1, -1
+        else:
+            from_port, to_port = parsed_args.dst_port
         data = compute_client.security_group_rules.create(
             group.id,
             parsed_args.proto,
@@ -314,48 +331,17 @@ class DeleteSecurityGroupRule(command.Command):
     def get_parser(self, prog_name):
         parser = super(DeleteSecurityGroupRule, self).get_parser(prog_name)
         parser.add_argument(
-            'group',
-            metavar='<group>',
-            help='Create rule in this security group',
-        )
-        parser.add_argument(
-            "--proto",
-            metavar="<proto>",
-            default="tcp",
-            help="IP protocol (icmp, tcp, udp; default: tcp)",
-        )
-        parser.add_argument(
-            "--src-ip",
-            metavar="<ip-address>",
-            default="0.0.0.0/0",
-            help="Source IP (may use CIDR notation; default: 0.0.0.0/0)",
-        )
-        parser.add_argument(
-            "--dst-port",
-            metavar="<port-range>",
-            action=parseractions.RangeAction,
-            help="Destination port, may be a range: 137:139 (default: 0; "
-                 "only required for proto tcp and udp)",
+            'rule',
+            metavar='<rule>',
+            help='Security group rule ID to delete',
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         compute_client = self.app.client_manager.compute
-        group = utils.find_resource(
-            compute_client.security_groups,
-            parsed_args.group,
-        )
-        from_port, to_port = parsed_args.dst_port
-        # sigh...delete by ID?
-        compute_client.security_group_rules.delete(
-            group.id,
-            parsed_args.proto,
-            from_port,
-            to_port,
-            parsed_args.src_ip,
-        )
+        compute_client.security_group_rules.delete(parsed_args.rule)
         return
 
 
@@ -369,7 +355,7 @@ class ListSecurityGroupRule(lister.Lister):
         parser.add_argument(
             'group',
             metavar='<group>',
-            help='Create rule in this security group',
+            help='List all rules in this security group',
         )
         return parser
 
