@@ -21,9 +21,11 @@ import six
 from cliff import command
 from cliff import lister
 from cliff import show
+from keystoneclient import exceptions as ksc_exc
 
 from openstackclient.common import exceptions
 from openstackclient.common import utils
+from openstackclient.i18n import _  # noqa
 
 
 class AddRole(show.ShowOne):
@@ -36,22 +38,24 @@ class AddRole(show.ShowOne):
         parser.add_argument(
             'role',
             metavar='<role>',
-            help='Role name or ID to add to user')
+            help=_('Role to add to <project>:<user> (name or ID)'),
+        )
         parser.add_argument(
             '--project',
             metavar='<project>',
             required=True,
-            help='Include project (name or ID)',
+            help=_('Include <project> (name or ID)'),
         )
         parser.add_argument(
             '--user',
             metavar='<user>',
             required=True,
-            help='Name or ID of user to include')
+            help=_('Include <user> (name or ID)'),
+        )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         role = utils.find_resource(identity_client.roles, parsed_args.role)
         project = utils.find_resource(
@@ -79,14 +83,30 @@ class CreateRole(show.ShowOne):
         parser = super(CreateRole, self).get_parser(prog_name)
         parser.add_argument(
             'role_name',
-            metavar='<role-name>',
-            help='New role name')
+            metavar='<name>',
+            help=_('New role name'),
+        )
+        parser.add_argument(
+            '--or-show',
+            action='store_true',
+            help=_('Return existing role'),
+        )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
-        role = identity_client.roles.create(parsed_args.role_name)
+        try:
+            role = identity_client.roles.create(parsed_args.role_name)
+        except ksc_exc.Conflict as e:
+            if parsed_args.or_show:
+                role = utils.find_resource(
+                    identity_client.roles,
+                    parsed_args.role_name,
+                )
+                self.log.info('Returning existing role %s', role.name)
+            else:
+                raise e
 
         info = {}
         info.update(role._info)
@@ -94,29 +114,30 @@ class CreateRole(show.ShowOne):
 
 
 class DeleteRole(command.Command):
-    """Delete existing role"""
+    """Delete role(s)"""
 
     log = logging.getLogger(__name__ + '.DeleteRole')
 
     def get_parser(self, prog_name):
         parser = super(DeleteRole, self).get_parser(prog_name)
         parser.add_argument(
-            'role',
+            'roles',
             metavar='<role>',
-            help='Name or ID of role to delete',
+            nargs="+",
+            help=_('Role(s) to delete (name or ID)'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
-        role = utils.find_resource(
-            identity_client.roles,
-            parsed_args.role,
-        )
-
-        identity_client.roles.delete(role.id)
+        for role in parsed_args.roles:
+            role_obj = utils.find_resource(
+                identity_client.roles,
+                role,
+            )
+            identity_client.roles.delete(role_obj.id)
         return
 
 
@@ -125,10 +146,75 @@ class ListRole(lister.Lister):
 
     log = logging.getLogger(__name__ + '.ListRole')
 
+    def get_parser(self, prog_name):
+        parser = super(ListRole, self).get_parser(prog_name)
+        parser.add_argument(
+            '--project',
+            metavar='<project>',
+            help='Filter roles by <project> (name or ID)',
+        )
+        parser.add_argument(
+            '--user',
+            metavar='<user>',
+            help='Filter roles by <user> (name or ID)',
+        )
+        return parser
+
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
-        columns = ('ID', 'Name')
-        data = self.app.client_manager.identity.roles.list()
+        identity_client = self.app.client_manager.identity
+        auth_ref = self.app.client_manager.auth_ref
+
+        # No user or project specified, list all roles in the system
+        if not parsed_args.user and not parsed_args.project:
+            columns = ('ID', 'Name')
+            data = identity_client.roles.list()
+        elif parsed_args.user and parsed_args.project:
+            user = utils.find_resource(
+                identity_client.users,
+                parsed_args.user,
+            )
+            project = utils.find_resource(
+                identity_client.projects,
+                parsed_args.project,
+            )
+            data = identity_client.roles.roles_for_user(user.id, project.id)
+
+        elif parsed_args.user:
+            user = utils.find_resource(
+                identity_client.users,
+                parsed_args.user,
+            )
+            if self.app.client_manager.auth_ref:
+                project = utils.find_resource(
+                    identity_client.projects,
+                    auth_ref.project_id
+                )
+            else:
+                msg = _("Project must be specified")
+                raise exceptions.CommandError(msg)
+            data = identity_client.roles.roles_for_user(user.id, project.id)
+        elif parsed_args.project:
+            project = utils.find_resource(
+                identity_client.projects,
+                parsed_args.project,
+            )
+            if self.app.client_manager.auth_ref:
+                user = utils.find_resource(
+                    identity_client.users,
+                    auth_ref.user_id
+                )
+            else:
+                msg = _("User must be specified")
+                raise exceptions.CommandError(msg)
+            data = identity_client.roles.roles_for_user(user.id, project.id)
+
+        if parsed_args.user or parsed_args.project:
+            columns = ('ID', 'Name', 'Project', 'User')
+            for user_role in data:
+                user_role.user = user.name
+                user_role.project = project.name
+
         return (columns,
                 (utils.get_item_properties(
                     s, columns,
@@ -147,16 +233,17 @@ class ListUserRole(lister.Lister):
             'user',
             metavar='<user>',
             nargs='?',
-            help='Name or ID of user to include')
+            help=_('User to list (name or ID)'),
+        )
         parser.add_argument(
             '--project',
             metavar='<project>',
-            help='Include project (name or ID)',
+            help=_('Filter users by <project> (name or ID)'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         auth_ref = self.app.client_manager.auth_ref
 
@@ -167,13 +254,13 @@ class ListUserRole(lister.Lister):
             if self.app.client_manager.auth_ref:
                 parsed_args.project = auth_ref.project_id
             else:
-                msg = "Project must be specified"
+                msg = _("Project must be specified")
                 raise exceptions.CommandError(msg)
         if not parsed_args.user:
             if self.app.client_manager.auth_ref:
                 parsed_args.user = auth_ref.user_id
             else:
-                msg = "User must be specified"
+                msg = _("User must be specified")
                 raise exceptions.CommandError(msg)
 
         project = utils.find_resource(
@@ -204,7 +291,7 @@ class ListUserRole(lister.Lister):
 
 
 class RemoveRole(command.Command):
-    """Remove role from project:user"""
+    """Remove role from project : user"""
 
     log = logging.getLogger(__name__ + '.RemoveRole')
 
@@ -213,22 +300,24 @@ class RemoveRole(command.Command):
         parser.add_argument(
             'role',
             metavar='<role>',
-            help='Role name or ID to remove from user')
+            help=_('Role to remove (name or ID)'),
+        )
         parser.add_argument(
             '--project',
             metavar='<project>',
             required=True,
-            help='Project to include (name or ID)',
+            help=_('Include <project> (name or ID)'),
         )
         parser.add_argument(
             '--user',
             metavar='<user>',
             required=True,
-            help='Name or ID of user')
+            help=_('Include <user> (name or ID)'),
+        )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         role = utils.find_resource(identity_client.roles, parsed_args.role)
         project = utils.find_resource(
@@ -243,7 +332,7 @@ class RemoveRole(command.Command):
 
 
 class ShowRole(show.ShowOne):
-    """Show single role"""
+    """Display role details"""
 
     log = logging.getLogger(__name__ + '.ShowRole')
 
@@ -252,11 +341,12 @@ class ShowRole(show.ShowOne):
         parser.add_argument(
             'role',
             metavar='<role>',
-            help='Name or ID of role to display')
+            help=_('Role to display (name or ID)'),
+        )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         role = utils.find_resource(identity_client.roles, parsed_args.role)
 
