@@ -15,6 +15,7 @@
 
 """Service action implementations"""
 
+import argparse
 import logging
 import six
 
@@ -24,6 +25,7 @@ from cliff import show
 
 from openstackclient.common import exceptions
 from openstackclient.common import utils
+from openstackclient.i18n import _  # noqa
 from openstackclient.identity import common
 
 
@@ -35,30 +37,54 @@ class CreateService(show.ShowOne):
     def get_parser(self, prog_name):
         parser = super(CreateService, self).get_parser(prog_name)
         parser.add_argument(
-            'name',
-            metavar='<service-name>',
-            help='New service name',
+            'type_or_name',
+            metavar='<type>',
+            help=_('New service type (compute, image, identity, volume, etc)'),
         )
-        parser.add_argument(
+        type_or_name_group = parser.add_mutually_exclusive_group()
+        type_or_name_group.add_argument(
             '--type',
-            metavar='<service-type>',
-            required=True,
-            help='New service type (compute, image, identity, volume, etc)',
+            metavar='<type>',
+            help=argparse.SUPPRESS,
+        )
+        type_or_name_group.add_argument(
+            '--name',
+            metavar='<name>',
+            help=_('New service name'),
         )
         parser.add_argument(
             '--description',
-            metavar='<service-description>',
-            help='New service description',
+            metavar='<description>',
+            help=_('New service description'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
+        type_or_name = parsed_args.type_or_name
+        name = parsed_args.name
+        type = parsed_args.type
+
+        # If only a single positional is present, it's a <type>.
+        # This is not currently legal so it is considered a new case.
+        if not type and not name:
+            type = type_or_name
+        # If --type option is present then positional is handled as <name>;
+        # display deprecation message.
+        elif type:
+            name = type_or_name
+            self.log.warning(_('The argument --type is deprecated, use service'
+                               ' create --name <service-name> type instead.'))
+        # If --name option is present the positional is handled as <type>.
+        # Making --type optional is new, but back-compatible
+        elif name:
+            type = type_or_name
+
         service = identity_client.services.create(
-            parsed_args.name,
-            parsed_args.type,
+            name,
+            type,
             parsed_args.description)
 
         info = {}
@@ -76,12 +102,12 @@ class DeleteService(command.Command):
         parser.add_argument(
             'service',
             metavar='<service>',
-            help='Service to delete (name or ID)',
+            help=_('Service to delete (name or ID)'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         service = common.find_service(identity_client, parsed_args.service)
         identity_client.services.delete(service.id)
@@ -99,26 +125,26 @@ class ListService(lister.Lister):
             '--long',
             action='store_true',
             default=False,
-            help='List additional fields in output')
+            help=_('List additional fields in output'),
+        )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         if parsed_args.long:
             columns = ('ID', 'Name', 'Type', 'Description')
         else:
-            columns = ('ID', 'Name')
+            columns = ('ID', 'Name', 'Type')
         data = self.app.client_manager.identity.services.list()
-        return (columns,
-                (utils.get_item_properties(
-                    s, columns,
-                    formatters={},
-                ) for s in data))
+        return (
+            columns,
+            (utils.get_item_properties(s, columns) for s in data),
+        )
 
 
 class ShowService(show.ShowOne):
-    """Show service details"""
+    """Display service details"""
 
     log = logging.getLogger(__name__ + '.ShowService')
 
@@ -127,22 +153,23 @@ class ShowService(show.ShowOne):
         parser.add_argument(
             'service',
             metavar='<service>',
-            help='Service to display (type, name or ID)',
+            help=_('Service to display (type, name or ID)'),
         )
         parser.add_argument(
             '--catalog',
             action='store_true',
             default=False,
-            help='Show service catalog information',
+            help=_('Show service catalog information'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
+        auth_ref = self.app.client_manager.auth_ref
 
         if parsed_args.catalog:
-            endpoints = identity_client.service_catalog.get_endpoints(
+            endpoints = auth_ref.service_catalog.get_endpoints(
                 service_type=parsed_args.service)
             for (service, service_endpoints) in six.iteritems(endpoints):
                 if service_endpoints:
@@ -150,8 +177,8 @@ class ShowService(show.ShowOne):
                     info.update(service_endpoints[0])
                     return zip(*sorted(six.iteritems(info)))
 
-            msg = ("No service catalog with a type, name or ID of '%s' "
-                   "exists." % (parsed_args.service))
+            msg = _("No service catalog with a type, name or ID of '%s' "
+                    "exists.") % (parsed_args.service)
             raise exceptions.CommandError(msg)
         else:
             service = common.find_service(identity_client, parsed_args.service)
