@@ -21,8 +21,10 @@ import six
 from cliff import command
 from cliff import lister
 from cliff import show
+from keystoneclient import exceptions as ksc_exc
 
 from openstackclient.common import utils
+from openstackclient.i18n import _  # noqa
 
 
 class CreateUser(show.ShowOne):
@@ -34,45 +36,50 @@ class CreateUser(show.ShowOne):
         parser = super(CreateUser, self).get_parser(prog_name)
         parser.add_argument(
             'name',
-            metavar='<user-name>',
-            help='New user name',
+            metavar='<name>',
+            help=_('New user name'),
+        )
+        parser.add_argument(
+            '--project',
+            metavar='<project>',
+            help=_('Default project (name or ID)'),
         )
         parser.add_argument(
             '--password',
-            metavar='<user-password>',
-            help='New user password',
+            metavar='<password>',
+            help=_('Set user password'),
         )
         parser.add_argument(
             '--password-prompt',
             dest="password_prompt",
             action="store_true",
-            help='Prompt interactively for password',
+            help=_('Prompt interactively for password'),
         )
         parser.add_argument(
             '--email',
-            metavar='<user-email>',
-            help='New user email address',
-        )
-        parser.add_argument(
-            '--project',
-            metavar='<project>',
-            help='Set default project (name or ID)',
+            metavar='<email-address>',
+            help=_('Set user email address'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
             action='store_true',
-            help='Enable user (default)',
+            help=_('Enable user (default)'),
         )
         enable_group.add_argument(
             '--disable',
             action='store_true',
-            help='Disable user',
+            help=_('Disable user'),
+        )
+        parser.add_argument(
+            '--or-show',
+            action='store_true',
+            help=_('Return existing user'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         if parsed_args.project:
@@ -89,19 +96,31 @@ class CreateUser(show.ShowOne):
         if parsed_args.password_prompt:
             parsed_args.password = utils.get_password(self.app.stdin)
 
-        user = identity_client.users.create(
-            parsed_args.name,
-            parsed_args.password,
-            parsed_args.email,
-            tenant_id=project_id,
-            enabled=enabled,
-        )
+        try:
+            user = identity_client.users.create(
+                parsed_args.name,
+                parsed_args.password,
+                parsed_args.email,
+                tenant_id=project_id,
+                enabled=enabled,
+            )
+        except ksc_exc.Conflict as e:
+            if parsed_args.or_show:
+                user = utils.find_resource(
+                    identity_client.users,
+                    parsed_args.name,
+                )
+                self.log.info('Returning existing user %s', user.name)
+            else:
+                raise e
+
         # NOTE(dtroyer): The users.create() method wants 'tenant_id' but
         #                the returned resource has 'tenantId'.  Sigh.
         #                We're using project_id now inside OSC so there.
-        user._info.update(
-            {'project_id': user._info.pop('tenantId')}
-        )
+        if 'tenantId' in user._info:
+            user._info.update(
+                {'project_id': user._info.pop('tenantId')}
+            )
 
         info = {}
         info.update(user._info)
@@ -109,29 +128,30 @@ class CreateUser(show.ShowOne):
 
 
 class DeleteUser(command.Command):
-    """Delete user"""
+    """Delete user(s)"""
 
     log = logging.getLogger(__name__ + '.DeleteUser')
 
     def get_parser(self, prog_name):
         parser = super(DeleteUser, self).get_parser(prog_name)
         parser.add_argument(
-            'user',
+            'users',
             metavar='<user>',
-            help='User to delete (name or ID)',
+            nargs="+",
+            help=_('User(s) to delete (name or ID)'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
-        user = utils.find_resource(
-            identity_client.users,
-            parsed_args.user,
-        )
-
-        identity_client.users.delete(user.id)
+        for user in parsed_args.users:
+            user_obj = utils.find_resource(
+                identity_client.users,
+                user,
+            )
+            identity_client.users.delete(user_obj.id)
         return
 
 
@@ -145,17 +165,17 @@ class ListUser(lister.Lister):
         parser.add_argument(
             '--project',
             metavar='<project>',
-            help='Filter users by project (name or ID)',
+            help=_('Filter users by project (name or ID)'),
         )
         parser.add_argument(
             '--long',
             action='store_true',
             default=False,
-            help='List additional fields in output')
+            help=_('List additional fields in output'))
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         def _format_project(project):
@@ -235,49 +255,49 @@ class SetUser(command.Command):
         parser.add_argument(
             'user',
             metavar='<user>',
-            help='User to change (name or ID)',
+            help=_('User to change (name or ID)'),
         )
         parser.add_argument(
             '--name',
-            metavar='<new-user-name>',
-            help='New user name',
+            metavar='<name>',
+            help=_('Set user name'),
+        )
+        parser.add_argument(
+            '--project',
+            metavar='<project>',
+            help=_('Set default project (name or ID)'),
         )
         parser.add_argument(
             '--password',
             metavar='<user-password>',
-            help='New user password',
+            help=_('Set user password'),
         )
         parser.add_argument(
             '--password-prompt',
             dest="password_prompt",
             action="store_true",
-            help='Prompt interactively for password',
+            help=_('Prompt interactively for password'),
         )
         parser.add_argument(
             '--email',
-            metavar='<user-email>',
-            help='New user email address',
-        )
-        parser.add_argument(
-            '--project',
-            metavar='<project>',
-            help='New default project (name or ID)',
+            metavar='<email-address>',
+            help=_('Set user email address'),
         )
         enable_group = parser.add_mutually_exclusive_group()
         enable_group.add_argument(
             '--enable',
             action='store_true',
-            help='Enable user (default)',
+            help=_('Enable user (default)'),
         )
         enable_group.add_argument(
             '--disable',
             action='store_true',
-            help='Disable user',
+            help=_('Disable user'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         if parsed_args.password_prompt:
@@ -329,7 +349,7 @@ class SetUser(command.Command):
 
 
 class ShowUser(show.ShowOne):
-    """Show user details"""
+    """Display user details"""
 
     log = logging.getLogger(__name__ + '.ShowUser')
 
@@ -338,28 +358,45 @@ class ShowUser(show.ShowOne):
         parser.add_argument(
             'user',
             metavar='<user>',
-            help='User to display (name or ID)',
+            help=_('User to display (name or ID)'),
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
-        user = utils.find_resource(
-            identity_client.users,
-            parsed_args.user,
-        )
-
-        if 'tenantId' in user._info:
-            user._info.update(
-                {'project_id': user._info.pop('tenantId')}
-            )
-        if 'tenant_id' in user._info:
-            user._info.update(
-                {'project_id': user._info.pop('tenant_id')}
-            )
-
         info = {}
-        info.update(user._info)
+        try:
+            user = utils.find_resource(
+                identity_client.users,
+                parsed_args.user,
+            )
+            info.update(user._info)
+        except ksc_exc.Forbidden as e:
+            auth_ref = self.app.client_manager.auth_ref
+            if (
+                parsed_args.user == auth_ref.user_id or
+                parsed_args.user == auth_ref.username
+            ):
+                # Ask for currently auth'ed project so return it
+                info = {
+                    'id': auth_ref.user_id,
+                    'name': auth_ref.username,
+                    'project_id': auth_ref.project_id,
+                    # True because we don't get this far if it is disabled
+                    'enabled': True,
+                }
+            else:
+                raise e
+
+        if 'tenantId' in info:
+            info.update(
+                {'project_id': info.pop('tenantId')}
+            )
+        if 'tenant_id' in info:
+            info.update(
+                {'project_id': info.pop('tenant_id')}
+            )
+
         return zip(*sorted(six.iteritems(info)))
