@@ -27,7 +27,7 @@ from openstackclient.common import utils
 
 
 class CreateConsumer(show.ShowOne):
-    """Create consumer command"""
+    """Create new consumer"""
 
     log = logging.getLogger(__name__ + '.CreateConsumer')
 
@@ -35,24 +35,23 @@ class CreateConsumer(show.ShowOne):
         parser = super(CreateConsumer, self).get_parser(prog_name)
         parser.add_argument(
             '--description',
-            metavar='<consumer-description>',
+            metavar='<description>',
             help='New consumer description',
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         consumer = identity_client.oauth1.consumers.create(
             parsed_args.description
         )
-        info = {}
-        info.update(consumer._info)
-        return zip(*sorted(six.iteritems(info)))
+        consumer._info.pop('links', None)
+        return zip(*sorted(six.iteritems(consumer._info)))
 
 
 class DeleteConsumer(command.Command):
-    """Delete consumer command"""
+    """Delete consumer"""
 
     log = logging.getLogger(__name__ + '.DeleteConsumer')
 
@@ -61,12 +60,12 @@ class DeleteConsumer(command.Command):
         parser.add_argument(
             'consumer',
             metavar='<consumer>',
-            help='ID of consumer to delete',
+            help='Consumer to delete',
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         consumer = utils.find_resource(
             identity_client.oauth1.consumers, parsed_args.consumer)
@@ -75,12 +74,12 @@ class DeleteConsumer(command.Command):
 
 
 class ListConsumer(lister.Lister):
-    """List consumer command"""
+    """List consumers"""
 
     log = logging.getLogger(__name__ + '.ListConsumer')
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         columns = ('ID', 'Description')
         data = self.app.client_manager.identity.oauth1.consumers.list()
         return (columns,
@@ -91,7 +90,7 @@ class ListConsumer(lister.Lister):
 
 
 class SetConsumer(command.Command):
-    """Set consumer command"""
+    """Set consumer properties"""
 
     log = logging.getLogger(__name__ + '.SetConsumer')
 
@@ -100,17 +99,17 @@ class SetConsumer(command.Command):
         parser.add_argument(
             'consumer',
             metavar='<consumer>',
-            help='ID of consumer to change',
+            help='Consumer to modify',
         )
         parser.add_argument(
             '--description',
-            metavar='<new-consumer-description>',
+            metavar='<description>',
             help='New consumer description',
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         consumer = utils.find_resource(
             identity_client.oauth1.consumers, parsed_args.consumer)
@@ -119,7 +118,7 @@ class SetConsumer(command.Command):
             kwargs['description'] = parsed_args.description
 
         if not len(kwargs):
-            sys.stdout.write("Consumer not updated, no arguments present")
+            sys.stdout.write('Consumer not updated, no arguments present')
             return
 
         consumer = identity_client.oauth1.consumers.update(
@@ -128,7 +127,7 @@ class SetConsumer(command.Command):
 
 
 class ShowConsumer(show.ShowOne):
-    """Show consumer command"""
+    """Display consumer details"""
 
     log = logging.getLogger(__name__ + '.ShowConsumer')
 
@@ -137,16 +136,15 @@ class ShowConsumer(show.ShowOne):
         parser.add_argument(
             'consumer',
             metavar='<consumer>',
-            help='ID of consumer to display',
+            help='Consumer to display',
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         consumer = utils.find_resource(
             identity_client.oauth1.consumers, parsed_args.consumer)
 
-        info = {}
-        info.update(consumer._info)
-        return zip(*sorted(six.iteritems(info)))
+        consumer._info.pop('links', None)
+        return zip(*sorted(six.iteritems(consumer._info)))
