@@ -57,8 +57,8 @@ class CreateCredential(show.ShowOne):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         user_id = utils.find_resource(identity_client.users,
                                       parsed_args.user).id
@@ -73,6 +73,7 @@ class CreateCredential(show.ShowOne):
             blob=parsed_args.data,
             project=project)
 
+        credential._info.pop('links')
         return zip(*sorted(six.iteritems(credential._info)))
 
 
@@ -90,8 +91,8 @@ class DeleteCredential(command.Command):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         identity_client.credentials.delete(parsed_args.credential)
         return
@@ -102,11 +103,12 @@ class ListCredential(lister.Lister):
 
     log = logging.getLogger(__name__ + '.ListCredential')
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
-        columns = ('ID', 'Type', 'User ID', 'Data', 'Project ID')
+        columns = ('ID', 'Type', 'User ID', 'Blob', 'Project ID')
+        column_headers = ('ID', 'Type', 'User ID', 'Data', 'Project ID')
         data = self.app.client_manager.identity.credentials.list()
-        return (columns,
+        return (column_headers,
                 (utils.get_item_properties(
                     s, columns,
                     formatters={},
@@ -148,14 +150,15 @@ class SetCredential(command.Command):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
-        user_id = utils.find_resource(identity_client.users,
-                                      parsed_args.user).id
         kwargs = {}
-        if user_id:
-            kwargs['user'] = user_id
+        if parsed_args.user:
+            user_id = utils.find_resource(identity_client.users,
+                                          parsed_args.user).id
+            if user_id:
+                kwargs['user'] = user_id
         if parsed_args.type:
             kwargs['type'] = parsed_args.type
         if parsed_args.data:
@@ -186,10 +189,11 @@ class ShowCredential(show.ShowOne):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
         credential = utils.find_resource(identity_client.credentials,
                                          parsed_args.credential)
 
+        credential._info.pop('links')
         return zip(*sorted(six.iteritems(credential._info)))
