@@ -19,11 +19,68 @@
 import logging
 import six
 
+from cliff import command
 from cliff import lister
 from cliff import show
 
 from openstackclient.common import utils
-from openstackclient.object.v1.lib import container as lib_container
+
+
+class CreateContainer(lister.Lister):
+    """Create new container"""
+
+    log = logging.getLogger(__name__ + '.CreateContainer')
+
+    def get_parser(self, prog_name):
+        parser = super(CreateContainer, self).get_parser(prog_name)
+        parser.add_argument(
+            'containers',
+            metavar='<container-name>',
+            nargs="+",
+            help='New container name(s)',
+        )
+        return parser
+
+    @utils.log_method(log)
+    def take_action(self, parsed_args):
+
+        results = []
+        for container in parsed_args.containers:
+            data = self.app.client_manager.object_store.container_create(
+                container=container,
+            )
+            results.append(data)
+
+        columns = ("account", "container", "x-trans-id")
+        return (columns,
+                (utils.get_dict_properties(
+                    s, columns,
+                    formatters={},
+                ) for s in results))
+
+
+class DeleteContainer(command.Command):
+    """Delete container"""
+
+    log = logging.getLogger(__name__ + '.DeleteContainer')
+
+    def get_parser(self, prog_name):
+        parser = super(DeleteContainer, self).get_parser(prog_name)
+        parser.add_argument(
+            'containers',
+            metavar='<container>',
+            nargs="+",
+            help='Container(s) to delete',
+        )
+        return parser
+
+    @utils.log_method(log)
+    def take_action(self, parsed_args):
+
+        for container in parsed_args.containers:
+            self.app.client_manager.object_store.container_delete(
+                container=container,
+            )
 
 
 class ListContainer(lister.Lister):
@@ -68,8 +125,8 @@ class ListContainer(lister.Lister):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         if parsed_args.long:
             columns = ('Name', 'Bytes', 'Count')
@@ -88,9 +145,7 @@ class ListContainer(lister.Lister):
         if parsed_args.all:
             kwargs['full_listing'] = True
 
-        data = lib_container.list_containers(
-            self.app.restapi,
-            self.app.client_manager.object_store.endpoint,
+        data = self.app.client_manager.object_store.container_list(
             **kwargs
         )
 
@@ -101,8 +156,30 @@ class ListContainer(lister.Lister):
                 ) for s in data))
 
 
+class SaveContainer(command.Command):
+    """Save container contents locally"""
+
+    log = logging.getLogger(__name__ + ".SaveContainer")
+
+    def get_parser(self, prog_name):
+        parser = super(SaveContainer, self).get_parser(prog_name)
+        parser.add_argument(
+            'container',
+            metavar='<container>',
+            help='Container to save',
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug("take_action(%s)", parsed_args)
+
+        self.app.client_manager.object_store.container_save(
+            container=parsed_args.container,
+        )
+
+
 class ShowContainer(show.ShowOne):
-    """Show container information"""
+    """Display container details"""
 
     log = logging.getLogger(__name__ + '.ShowContainer')
 
@@ -111,17 +188,15 @@ class ShowContainer(show.ShowOne):
         parser.add_argument(
             'container',
             metavar='<container>',
-            help='Container name to display',
+            help='Container to display',
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
-        data = lib_container.show_container(
-            self.app.restapi,
-            self.app.client_manager.object_store.endpoint,
-            parsed_args.container,
+        data = self.app.client_manager.object_store.container_show(
+            container=parsed_args.container,
         )
 
         return zip(*sorted(six.iteritems(data)))
