@@ -19,11 +19,80 @@
 import logging
 import six
 
+from cliff import command
 from cliff import lister
 from cliff import show
 
 from openstackclient.common import utils
-from openstackclient.object.v1.lib import object as lib_object
+
+
+class CreateObject(lister.Lister):
+    """Upload object to container"""
+
+    log = logging.getLogger(__name__ + '.CreateObject')
+
+    def get_parser(self, prog_name):
+        parser = super(CreateObject, self).get_parser(prog_name)
+        parser.add_argument(
+            'container',
+            metavar='<container>',
+            help='Container for new object',
+        )
+        parser.add_argument(
+            'objects',
+            metavar='<filename>',
+            nargs="+",
+            help='Local filename(s) to upload',
+        )
+        return parser
+
+    @utils.log_method(log)
+    def take_action(self, parsed_args):
+
+        results = []
+        for obj in parsed_args.objects:
+            data = self.app.client_manager.object_store.object_create(
+                container=parsed_args.container,
+                object=obj,
+            )
+            results.append(data)
+
+        columns = ("object", "container", "etag")
+        return (columns,
+                (utils.get_dict_properties(
+                    s, columns,
+                    formatters={},
+                ) for s in results))
+
+
+class DeleteObject(command.Command):
+    """Delete object from container"""
+
+    log = logging.getLogger(__name__ + '.DeleteObject')
+
+    def get_parser(self, prog_name):
+        parser = super(DeleteObject, self).get_parser(prog_name)
+        parser.add_argument(
+            'container',
+            metavar='<container>',
+            help='Delete object(s) from <container>',
+        )
+        parser.add_argument(
+            'objects',
+            metavar='<object>',
+            nargs="+",
+            help='Object(s) to delete',
+        )
+        return parser
+
+    @utils.log_method(log)
+    def take_action(self, parsed_args):
+
+        for obj in parsed_args.objects:
+            self.app.client_manager.object_store.object_delete(
+                container=parsed_args.container,
+                object=obj,
+            )
 
 
 class ListObject(lister.Lister):
@@ -35,8 +104,8 @@ class ListObject(lister.Lister):
         parser = super(ListObject, self).get_parser(prog_name)
         parser.add_argument(
             "container",
-            metavar="<container-name>",
-            help="List contents of container-name",
+            metavar="<container>",
+            help="Container to list",
         )
         parser.add_argument(
             "--prefix",
@@ -78,8 +147,8 @@ class ListObject(lister.Lister):
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
         if parsed_args.long:
             columns = (
@@ -106,10 +175,8 @@ class ListObject(lister.Lister):
         if parsed_args.all:
             kwargs['full_listing'] = True
 
-        data = lib_object.list_objects(
-            self.app.restapi,
-            self.app.client_manager.object_store.endpoint,
-            parsed_args.container,
+        data = self.app.client_manager.object_store.object_list(
+            container=parsed_args.container,
             **kwargs
         )
 
@@ -120,8 +187,42 @@ class ListObject(lister.Lister):
                 ) for s in data))
 
 
+class SaveObject(command.Command):
+    """Save object locally"""
+
+    log = logging.getLogger(__name__ + ".SaveObject")
+
+    def get_parser(self, prog_name):
+        parser = super(SaveObject, self).get_parser(prog_name)
+        parser.add_argument(
+            "--file",
+            metavar="<filename>",
+            help="Destination filename (defaults to object name)",
+        )
+        parser.add_argument(
+            'container',
+            metavar='<container>',
+            help='Download <object> from <container>',
+        )
+        parser.add_argument(
+            "object",
+            metavar="<object>",
+            help="Object to save",
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug("take_action(%s)", parsed_args)
+
+        self.app.client_manager.object_store.object_save(
+            container=parsed_args.container,
+            object=parsed_args.object,
+            file=parsed_args.file,
+        )
+
+
 class ShowObject(show.ShowOne):
-    """Show object information"""
+    """Display object details"""
 
     log = logging.getLogger(__name__ + '.ShowObject')
 
@@ -130,23 +231,21 @@ class ShowObject(show.ShowOne):
         parser.add_argument(
             'container',
             metavar='<container>',
-            help='Container name for object to display',
+            help='Display <object> from <container>',
         )
         parser.add_argument(
             'object',
             metavar='<object>',
-            help='Object name to display',
+            help='Object to display',
         )
         return parser
 
+    @utils.log_method(log)
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)', parsed_args)
 
-        data = lib_object.show_object(
-            self.app.restapi,
-            self.app.client_manager.object_store.endpoint,
-            parsed_args.container,
-            parsed_args.object,
+        data = self.app.client_manager.object_store.object_show(
+            container=parsed_args.container,
+            object=parsed_args.object,
         )
 
         return zip(*sorted(six.iteritems(data)))
