@@ -16,7 +16,9 @@
 import mock
 
 from openstackclient.tests import fakes
+from openstackclient.tests.identity.v2_0 import fakes as identity_fakes
 from openstackclient.tests.image.v2 import fakes as image_fakes
+from openstackclient.tests.network.v2 import fakes as network_fakes
 from openstackclient.tests import utils
 
 
@@ -26,6 +28,7 @@ server_name = 'waiter'
 SERVER = {
     'id': server_id,
     'name': server_name,
+    'metadata': {},
 }
 
 extension_name = 'Multinic'
@@ -47,6 +50,34 @@ EXTENSION = {
     'links': extension_links,
 }
 
+flavor_id = 'm1.large'
+flavor_name = 'Large'
+flavor_ram = 8192
+flavor_vcpus = 4
+
+FLAVOR = {
+    'id': flavor_id,
+    'name': flavor_name,
+    'ram': flavor_ram,
+    'vcpus': flavor_vcpus,
+}
+
+floating_ip_num = 100
+fix_ip_num = 100
+injected_file_num = 100
+key_pair_num = 100
+project_name = 'project_test'
+QUOTA = {
+    'project': project_name,
+    'floating-ips': floating_ip_num,
+    'fix-ips': fix_ip_num,
+    'injected-files': injected_file_num,
+    'key-pairs': key_pair_num,
+}
+
+QUOTA_columns = tuple(sorted(QUOTA))
+QUOTA_data = tuple(QUOTA[x] for x in sorted(QUOTA))
+
 
 class FakeComputev2Client(object):
     def __init__(self, **kwargs):
@@ -56,6 +87,10 @@ class FakeComputev2Client(object):
         self.servers.resource_class = fakes.FakeResource(None, {})
         self.extensions = mock.Mock()
         self.extensions.resource_class = fakes.FakeResource(None, {})
+        self.flavors = mock.Mock()
+        self.flavors.resource_class = fakes.FakeResource(None, {})
+        self.quotas = mock.Mock()
+        self.quotas.resource_class = fakes.FakeResource(None, {})
         self.auth_token = kwargs['token']
         self.management_url = kwargs['endpoint']
 
@@ -69,7 +104,17 @@ class TestComputev2(utils.TestCommand):
             token=fakes.AUTH_TOKEN,
         )
 
+        self.app.client_manager.identity = identity_fakes.FakeIdentityv2Client(
+            endpoint=fakes.AUTH_URL,
+            token=fakes.AUTH_TOKEN,
+        )
+
         self.app.client_manager.image = image_fakes.FakeImagev2Client(
             endpoint=fakes.AUTH_URL,
             token=fakes.AUTH_TOKEN,
         )
+
+        self.app.client_manager.network = network_fakes.FakeNetworkV2Client(
+            endpoint=fakes.AUTH_URL,
+            token=fakes.AUTH_TOKEN,
+        )
